import { Domain, DomainModule } from 'bf-types';
import { Nullable } from './types';
/**
 * Convert a block-5 domain name into a URI formatted string
 *
 * @param domainName The name of a block-5 domain
 */
export declare function domainToUri(domainName: Domain): string;
/**
 * Convert a block-5 module name into a URI formatted string
 *
 * @param moduleName The name of a block-5 module
 */
export declare function moduleToUri(moduleName: DomainModule): string;
/**
 * Get the domain name associated with a particular module name
 *
 * @param moduleName The name of a block-5 module
 */
export declare function getModuleDomain(moduleName: DomainModule): Nullable<Domain>;
/**
 * Get the base URI associated with a particular block-5 module name
 *
 * @param moduleName The name of a block-5 module
 */
export declare function getModuleBaseUri(moduleName: string, uri?: string): string;
/**
 * Ensure that both a domain name and module name are valid
 *
 * @param domainName The name of a block-5 domain
 * @param moduleName The name of a block-5 module
 */
export declare function validateDomainAndModule(domainName: Domain, moduleName: DomainModule): void;
