"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formTemplateToChanges = exports.resolveBracketSyntax = exports.castValueFromContext = exports.convertToTemplateBidningFromDb = exports.convertFormTemplateBindingFromDb = exports.resolveOutputBindingValue = exports.deepResolve = void 0;
const handlebars_1 = __importDefault(require("@timwoods/handlebars"));
const deepmerge_1 = __importDefault(require("deepmerge"));
const deepResolve = (fullkey, object) => {
    const keys = fullkey.split('.');
    let current = Object.assign({}, object);
    for (const key of keys) {
        if (typeof current === 'object' && current) {
            current = current[key];
        }
    }
    return current;
};
exports.deepResolve = deepResolve;
function combineMerge(target, source, options) {
    const destination = target.slice();
    source.forEach((item, index) => {
        if (typeof destination[index] === 'undefined') {
            destination[index] = options.cloneUnlessOtherwiseSpecified(item, options);
        }
        else if (options.isMergeableObject(item)) {
            destination[index] = deepmerge_1.default(target[index], item, options);
        }
        else if (target.indexOf(item) === -1) {
            destination.push(item);
        }
    });
    return destination;
}
const resolveOutputBindingValue = (binding, fullContext) => {
    switch (binding.type) {
        case 'CONTEXT':
            return exports.deepResolve(`${binding.value.data_source}.${binding.value.data_key}`, fullContext);
        case 'CONTEXT_TEMPLATE':
            return handlebars_1.default.compile(binding.value)(fullContext);
        case 'VALUE':
            return binding.value;
    }
};
exports.resolveOutputBindingValue = resolveOutputBindingValue;
const convertFormTemplateBindingFromDb = (formTemplate) => {
    const clean = {};
    for (const key of Object.keys(formTemplate)) {
        clean[key.replace(/,/g, '.')] = formTemplate[key];
    }
    return clean;
};
exports.convertFormTemplateBindingFromDb = convertFormTemplateBindingFromDb;
const convertToTemplateBidningFromDb = (formTemplate) => {
    const clean = {};
    for (const key of Object.keys(formTemplate)) {
        clean[key.replace(/\./g, ',')] = formTemplate[key];
    }
    return clean;
};
exports.convertToTemplateBidningFromDb = convertToTemplateBidningFromDb;
function castValueFromContext(contextKeyValue, matchValue) {
    switch (typeof contextKeyValue) {
        case 'number':
            return parseInt(matchValue, 10);
        case 'boolean': {
            if (matchValue === 'true') {
                return true;
            }
            else if (matchValue === 'false') {
                return false;
            }
        }
        default:
            return matchValue;
    }
}
exports.castValueFromContext = castValueFromContext;
function resolveBracketSyntax(target, keys, bindingValues, fullContext) {
    const preKey = keys.slice(0, -1).join('.');
    const postKey = keys[keys.length - 1].replace(/[\[\]]/g, '');
    const matchers = postKey.split(';').map((subMatcher) => subMatcher.split(':'));
    let contextValues = exports.deepResolve(`${target}.${preKey}`, fullContext) || [];
    bindingValues = Array.isArray(bindingValues) ? bindingValues : [];
    if (!Array.isArray(contextValues)) {
        contextValues = [];
    }
    return bindingValues.flatMap((bindingValue) => {
        return contextValues
            .filter((contextValue) => matchers.reduce((pv, cv) => {
            if (cv.length === 1) {
                return pv && bindingValue[cv[0]] === contextValue[cv[0]];
            }
            else if (cv.length === 2) {
                const contextKeyValue = contextValue[cv[0]];
                return pv && contextKeyValue === castValueFromContext(contextKeyValue, cv[1]);
            }
            return pv && false;
        }, true))
            .map((filterValue) => {
            const key = matchers
                .map((matcher) => {
                if (matcher.length === 1) {
                    return [...matcher, filterValue[matcher[0]]].join(':');
                }
                return matcher.join(':');
            })
                .join(';');
            return {
                [`${preKey}.[${key}]`]: deepmerge_1.default(filterValue, bindingValue, {
                    arrayMerge: combineMerge,
                }),
            };
        });
    });
}
exports.resolveBracketSyntax = resolveBracketSyntax;
const formTemplateToChanges = (form_data, context, formTemplate) => {
    var _a, _b;
    const fullContext = Object.assign(Object.assign({}, context), { form_data });
    const changes = [];
    const form_output_bindings = exports.convertFormTemplateBindingFromDb(formTemplate.form_output_bindings);
    const entity = {
        module: {
            module_name: ((_a = context.entity) === null || _a === void 0 ? void 0 : _a.module_name) || formTemplate.linked_module_name,
            module_id: ((_b = context.entity) === null || _b === void 0 ? void 0 : _b.id) || 'create',
        },
        changes: {},
    };
    const creator = {
        module: {
            module_name: context.creator.module_name,
            module_id: context.creator.id,
        },
        changes: {},
    };
    const actor = {
        module: {
            module_name: context.actor.module_name,
            module_id: context.actor.id,
        },
        changes: {},
    };
    function updateChanges(target, subkey, value) {
        switch (target) {
            case 'creator':
                creator.changes[subkey] = value;
                break;
            case 'entity':
                entity.changes[subkey] = value;
                break;
            case 'actor':
                actor.changes[subkey] = value;
                break;
        }
    }
    for (let option of Object.keys(form_output_bindings)) {
        option = option.trim();
        const binding = form_output_bindings[option];
        const value = exports.resolveOutputBindingValue(binding, fullContext);
        const keys = option.split('.');
        const target = keys.shift();
        const subkey = keys.join('.');
        if (/.*\[.+\]$/.test(option)) {
            const bracketSyntaxChanges = resolveBracketSyntax(target, keys, value, fullContext);
            for (const change of bracketSyntaxChanges) {
                for (const [key, value] of Object.entries(change)) {
                    updateChanges(target, key, value);
                }
            }
        }
        else {
            updateChanges(target, subkey, value);
        }
    }
    changes.push(entity);
    if (Object.keys(actor.changes).length !== 0) {
        changes.push(actor);
    }
    if (Object.keys(creator.changes).length !== 0) {
        changes.push(creator);
    }
    return changes;
};
exports.formTemplateToChanges = formTemplateToChanges;
